using System;
using System.Web;
using System.Configuration;

namespace SID.NHibernateUtils.MultipleSessions
{
   /// <summary>
   /// Implements the Open-Session-In-View pattern using <see cref="NHibernateSessionManager" />.
   /// Inspiration for this class came from Ed Courtenay at 
   /// http://sourceforge.net/forum/message.php?msg_id=2847509.
   /// </summary>
   public class NHibernateSessionModule : IHttpModule
   {
      public void Init(HttpApplication context)
      {
         context.BeginRequest += new EventHandler(BeginTransaction);
         context.EndRequest += new EventHandler(CommitAndCloseSession);
         context.PostRequestHandlerExecute += new EventHandler(FlushAllSessions);
      }

      public void Dispose() { }

      /// <summary>
      /// Opens a session within a transaction at the beginning of the HTTP request.  Note that 
      /// it ONLY begins transactions for those designated as being transactional.
      /// </summary>
      private void BeginTransaction(object sender, EventArgs e)
      {
         OpenSessionInViewSection openSessionInViewSection = GetOpenSessionInViewSection();

         if (openSessionInViewSection != null)
            foreach (SessionFactoryElement sessionFactorySettings in openSessionInViewSection.SessionFactories)
            {
               if (sessionFactorySettings.IsTransactional)
               {
                  NHibernateSessionManager.Instance.BeginTransactionOn(sessionFactorySettings.FactoryConfigPath);
               }
            }
      }

      /// <summary>
      /// Commits and closes the NHibernate session provided by the supplied <see cref="NHibernateSessionManager"/>.
      /// Assumes a transaction was begun at the beginning of the request; but a transaction or session does
      /// not *have* to be opened for this to operate successfully.
      /// </summary>
      private void CommitAndCloseSession(object sender, EventArgs e)
      {
         try
         {
            // Commit every session factory that's holding a transactional session
            NHibernateSessionManager.Instance.CommitAllTransactions();
         }
         finally
         {
            // No matter what happens, make sure all the sessions get closed
            NHibernateSessionManager.Instance.CloseAllSessions();
         }

         //OpenSessionInViewSection openSessionInViewSection = GetOpenSessionInViewSection();

         //try
         //{
         //   // Commit every session factory that's holding a transactional session
         //   foreach (SessionFactoryElement sessionFactorySettings in openSessionInViewSection.SessionFactories)
         //   {
         //      if (sessionFactorySettings.IsTransactional)
         //      {
         //         NHibernateSessionManager.Instance.CommitTransactionOn(sessionFactorySettings.FactoryConfigPath);
         //      }
         //   }
         //}
         //finally
         //{
         //   // No matter what happens, make sure all the sessions get closed
         //   foreach (SessionFactoryElement sessionFactorySettings in openSessionInViewSection.SessionFactories)
         //   {
         //      NHibernateSessionManager.Instance.CloseSessionOn(sessionFactorySettings.FactoryConfigPath);
         //   }
         //}
      }

      /// <summary>
      /// flush any nhib session to modify the status of object (if binded to asp.net session state)
      /// before they get persisted to aspnet session repository (inproc, outofprocess, sqlstate etc..)
      /// in such a way the object status in nhib is the same as in the session object otherwise we
      /// ca have a mismatch between the 2..ie object persisted in aspnet session state before nhib gets flushed
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void FlushAllSessions(object sender, EventArgs e)
      {
         NHibernateSessionManager.Instance.FlushAllSessions();
      }

      private OpenSessionInViewSection GetOpenSessionInViewSection()
      {
         //OpenSessionInViewSection openSessionInViewSection = ConfigurationSettings.GetConfig("nhibernateSettings") as OpenSessionInViewSection;

			OpenSessionInViewSection openSessionInViewSection = System.Configuration.ConfigurationManager.GetSection("nhibernateSettings") as OpenSessionInViewSection;

         //if (openSessionInViewSection == null)
         //   throw new ConfigurationErrorsException("The nhibernateSettings section was not found by ConfigurationManager.");

         return openSessionInViewSection;
      }
   }
}
